@@ -1,15 +1,20 @@
-from abc import ABCMeta, abstractmethod
+from __future__ import division
 import unittest
 import datetime
 import sys
 import mock
 import numpy
 import scipy
+from quantdsl.runtime import MultiProcessingDependencyGraphRunner, DependencyGraphRunner, DependencyGraph, \
+    SingleThreadedDependencyGraphRunner
 
-from quantdsl import DslParser, Number, Add, Max, On, Date, String, FunctionDef, \
-    Name, Compare, IfExp, FunctionCall, Sub, QuantDslSyntaxError, Mult, Div, UnarySub, Pow, Mod, FloorDiv, \
-    ExpressionStack, If, TimeDelta, Module, parse, compile, eval, utc, LeastSquares, DslNamespace, DslExpression, \
-    BlackScholesPriceProcess
+from quantdsl import utc
+from quantdsl.exceptions import DslSyntaxError
+from quantdsl.priceprocess.blackscholes import BlackScholesPriceProcess
+from quantdsl.semantics import DslExpression, String, Number, Date, TimeDelta, UnarySub, Add, Sub, Mult, Div, Pow, Mod, \
+    FloorDiv, Max, On, LeastSquares, FunctionCall, FunctionDef, Name, If, IfExp, Compare, Module, DslNamespace
+from quantdsl.services import eval, compile, parse
+from quantdsl.syntax import DslParser
 
 
 def suite():
@@ -23,8 +28,8 @@ class TestDslParser(unittest.TestCase):
 
     def test_empty_string(self):
         self.assertTrue(isinstance(parse(""), Module))
-        self.assertRaises(QuantDslSyntaxError, compile, "")
-        self.assertRaises(QuantDslSyntaxError, eval, "")
+        self.assertRaises(DslSyntaxError, compile, "")
+        self.assertRaises(DslSyntaxError, eval, "")
 
     def assertDslExprTypeValue(self, dslSource, expectedDslType, expectedDslValue, **compileKwds):
         # Assumes dslSource is just one statement.
@@ -70,7 +75,7 @@ class TestDslParser(unittest.TestCase):
         self.assertDslExprTypeValue("-Max(bar - 4, -9)", UnarySub, 8, bar=-4)
 
         # Check unsupported unary operators cause DSL errors.
-        self.assertRaises(QuantDslSyntaxError, parse, "~bar")
+        self.assertRaises(DslSyntaxError, parse, "~bar")
 
     def test_binop(self):
         self.assertDslExprTypeValue("5 + 2", Add, 7)
@@ -84,11 +89,11 @@ class TestDslParser(unittest.TestCase):
         self.assertDslExprTypeValue("5 % 2", Mod, 1)
 
         # Check unsupported binary operators cause DSL errors.
-        self.assertRaises(QuantDslSyntaxError, parse, "2 << 1")  # Bit shift left.
-        self.assertRaises(QuantDslSyntaxError, parse, "2 >> 1")  # Bit shift right.
-        self.assertRaises(QuantDslSyntaxError, parse, "2 & 1")  # Bitwise 'and'.
-        self.assertRaises(QuantDslSyntaxError, parse, "2 | 1")  # Complement
-        self.assertRaises(QuantDslSyntaxError, parse, "2 ^ 1")  # Bitwise exclusive or.
+        self.assertRaises(DslSyntaxError, parse, "2 << 1")  # Bit shift left.
+        self.assertRaises(DslSyntaxError, parse, "2 >> 1")  # Bit shift right.
+        self.assertRaises(DslSyntaxError, parse, "2 & 1")  # Bitwise 'and'.
+        self.assertRaises(DslSyntaxError, parse, "2 | 1")  # Complement
+        self.assertRaises(DslSyntaxError, parse, "2 ^ 1")  # Bitwise exclusive or.
 
     def test_compare(self):
         self.assertDslExprTypeValue("1 == 1", Compare, True)
@@ -391,7 +396,7 @@ sqr(3)
         self.assertEqual(dslExpr.evaluate(), 9)
 
         dslValue = eval(dslSource)
-        self.assertEqual(dslValue['mean'], 9)
+        self.assertEqual(dslValue, 9)
 
         # Expression with two function defs.
         dslSource = """
@@ -410,7 +415,7 @@ mul(3, 3)
         self.assertEqual(dslExpr.evaluate(), 9)
 
         dslValue = eval(dslSource)
-        self.assertEqual(dslValue['mean'], 9)
+        self.assertEqual(dslValue, 9)
 
 
     def test_parallel_fib(self):
@@ -435,10 +440,10 @@ fib(%d)
         dslExpr = compile(dslSource, isParallel=True)
 
         # Expect an expression stack object.
-        self.assertIsInstance(dslExpr, ExpressionStack)
+        self.assertIsInstance(dslExpr, DependencyGraph)
 
         # Remember the number of stubbed exprs - will check it after the value.
-        actualLenStubbedExprs = len(dslExpr.stubbedExprs)
+        actualLenStubbedExprs = len(dslExpr.stubbedExprsData)
 
         # Evaluate the stack.
         dslValue = dslExpr.evaluate()
@@ -450,7 +455,7 @@ fib(%d)
         self.assertEqual(actualLenStubbedExprs, expectedLenStubbedExprs)
 
         # Also check the runner call count is the same.
-        self.assertEqual(dslExpr._runnerCallCount, expectedLenStubbedExprs)
+        self.assertEqual(dslExpr.runner.callCount, expectedLenStubbedExprs)
 
     def test_parallel_american_option(self):
         # Branching function calls.
@@ -474,10 +479,10 @@ American(Date('2012-01-01'), Date('2012-01-03'), 5, 10, TimeDelta('1d'))
         dslExpr = compile(dslSource, isParallel=True)
 
         # Expect an expression stack object.
-        self.assertIsInstance(dslExpr, ExpressionStack)
+        self.assertIsInstance(dslExpr, DependencyGraph)
 
         # Remember the number of stubbed exprs - will check it after the value.
-        actualLenStubbedExprs = len(dslExpr.stubbedExprs)
+        actualLenStubbedExprs = len(dslExpr.stubbedExprsData)
 
         # Evaluate the stack.
         image = mock.Mock()
@@ -517,7 +522,7 @@ Swing(Date('2012-01-01'), Date('2012-01-03'), 10, 500)
         dslExpr = compile(dslSource, isParallel=True)
 
         # Remember the number of stubbed exprs - will check it after the value.
-        actualLenStubbedExprs = len(dslExpr.stubbedExprs)
+        actualLenStubbedExprs = len(dslExpr)
 
         # Evaluate the stack.
         image = mock.Mock()
@@ -527,6 +532,7 @@ Swing(Date('2012-01-01'), Date('2012-01-03'), 10, 500)
             'interestRate': 0,
             'presentTime': datetime.datetime(2011, 1, 1),
         }
+
         dslValue = dslExpr.evaluate(**kwds)
 
         # Check the value is expected.
@@ -554,10 +560,10 @@ Swing(Date('2011-01-01'), Date('2011-01-03'), 10, 500)
 """
 
         dslExpr = compile(dslSource, isParallel=True)
-        assert isinstance(dslExpr, ExpressionStack)
+        assert isinstance(dslExpr, DependencyGraph)
 
         # Remember the number of stubbed exprs - will check it after the value.
-        actualLenStubbedExprs = len(dslExpr.stubbedExprs)
+        actualLenStubbedExprs = len(dslExpr.stubbedExprsData)
 
         # Create a mock valuation environment.
         # image = mock.Mock()
@@ -575,7 +581,7 @@ Swing(Date('2011-01-01'), Date('2011-01-03'), 10, 500)
 
 
         # Evaluate the stack.
-        dslValue = dslExpr.evaluate(isMultiprocessing=True, **kwds).mean()
+        dslValue = dslExpr.evaluate(dependencyGraphRunnerClass=MultiProcessingDependencyGraphRunner, **kwds).mean()
 
         # Check the value is expected.
         self.assertEqual(dslValue, expectedValue)
@@ -697,7 +703,7 @@ class DslTestCase(unittest.TestCase):
                 '#2-LAST-PRICE': 10,
                 '#2-ACTUAL-HISTORICAL-VOLATILITY': 50,
             },
-            'allRvs': BlackScholesPriceProcess().getAllRvs(dslExpr, observationTime, pathCount=500000),
+            'allRvs': BlackScholesPriceProcess().getAllRvs(dslExpr, observationTime, pathCount=100000),
         })
         assert isinstance(dslExpr, DslExpression)
         value = dslExpr.evaluate(**evaluationKwds)
@@ -751,6 +757,10 @@ class TestDslMax(DslTestCase):
 
 class TestDslAdd(DslTestCase):
 
+    def testValuation(self):
+        specification = "10 + Market('#1')"
+        self.assertValuation(specification, 20, 1, 0)
+
     def testValuation(self):
         specification = "10 + Market('#2')"
         self.assertValuation(specification, 20, 1, 0)
@@ -889,7 +899,7 @@ Fixing( Date('2011-06-01'), Choice( Market('#1') - 9,
     Fixing( Date('2012-01-01'), Choice( Market('#1') - 9, 0))
 ))
 """
-        self.assertValuation(specification, 2.416, 0.677, 0.07)
+        self.assertValuation(specification, 2.401, 0.677, 0.0001)
 
 
 class TestDslSumContracts(DslTestCase):
@@ -922,12 +932,12 @@ Fixing(
     )
 )
 """
-        self.assertValuation(specification, 2 * 2.416, 2 * 0.677, 2*0.07, 0.04, 0.2, 0.2)
+        self.assertValuation(specification, 4.812, 2 * 0.677, 2*0.07, 0.04, 0.2, 0.2)
 
 
 class TestDslAddition(DslTestCase):
 
-    def testValuation(self):
+    def testValuation2(self):
         specification = """
 Fixing( Date('2012-01-01'),
     Max(Market('#1') - 9, 0) + Market('#1') - 9
